@@ -13,15 +13,15 @@ $SIG{CHLD}="IGNORE";
 
 =head1 NAME
 
-IPDR::Collection::CiscoSSL - IPDR Collection Client (Cisco Specification)
+IPDR::Collection::Cisco - IPDR Collection Client (Cisco Specification)
 
 =head1 VERSION
 
-Version 0.41
+Version 0.25
 
 =cut
 
-our $VERSION = '0.41';
+our $VERSION = '0.25';
 
 =head1 SYNOPSIS
 
@@ -313,9 +313,6 @@ sub new {
         if ( !$self->{_GLOBAL}{'DataHandler'} )
                 { die "DataHandler Function Must Be Defined"; }
 
-        if ( !$self->{_GLOBAL}{'PollTime'} )
-	                { $self->{_GLOBAL}{'PollTime'}=900 }
-
         $self->{_GLOBAL}{'handles'}= \%handles;
 	$self->{_GLOBAL}{'complete_decoded_data'} = \%complete_decoded_data;
 
@@ -385,8 +382,7 @@ foreach my $handle ( @{$current_handles} )
 					$self->{_GLOBAL}{'DataHandler'}->(
 						${$handles}{$handle}{'addr'},
 						${$handles}{$handle}{'port'},
-						${$handles}{$handle}{'data'},
-						$self );
+						${$handles}{$handle}{'data'} );
 					}
 				# remote sending needs to go here.
 				if ( $self->{_GLOBAL}{'RemoteIP'} && $self->{_GLOBAL}{'RemotePort'} )
@@ -419,12 +415,6 @@ foreach my $handle ( @{$current_handles} )
 return 1;
 }
 
-sub ReturnPollTime
-{
-my ( $self ) = shift;
-return $self->{_GLOBAL}{'PollTime'};
-}
-
 sub return_error
 {
 my ( $self ) = shift;
@@ -819,8 +809,6 @@ my ( %templates ) =
 		'Downstream-serviceSlaDropPkts' => 'SLAdropPkts',
 		'Upstream-serviceIdentifier' => 'SFID',
 		'Downstream-serviceIdentifier' => 'SFID',
-		'Upstream-serviceTimeActive' => 'serviceTimeActive',
-		'Downstream-serviceTimeActive' => 'serviceTimeActive',
 		'all-CMTShostname' => 'CMTShostname',
 		'all-CMdocsisMode' => 'CMdocsisMode',
 		'all-CMTSipAddress' => 'CMTSipAddress',
@@ -845,8 +833,6 @@ my ( %templates ) =
 		'Downstream-serviceSlaDelayPkts' => 'serviceSlaDelayPkts',
 		'Upstream-serviceSlaDropPkts' => 'serviceSlaDropPkts',
 		'Downstream-serviceSlaDropPkts' => 'serviceSlaDropPkts',
-		'Upstream-serviceTimeActive' => 'serviceTimeActive',
-		'Downstream-serviceTimeActive' => 'serviceTimeActive',
 		'all-CMTShostName' => 'CMTShostName',
 		'all-IPDRcreationTime' => 'IPDRcreationTime',
 		'all-RecType' => 'RecType',
@@ -874,8 +860,6 @@ my ( %templates ) =
 		'Downstream-serviceSlaDelayPkts' => 'serviceSlaDelayPkts',
 		'Upstream-serviceSlaDropPkts' => 'serviceSlaDropPkts',
 		'Downstream-serviceSlaDropPkts' => 'serviceSlaDropPkts',
-		'Upstream-serviceTimeActive' => 'serviceTimeActive',
-		'Downstream-serviceTimeActive' => 'serviceTimeActive',
 		'all-CMTShostName' => 'CMTShostName',
 		'all-IPDRcreationTime' => 'IPDRcreationTime',
 		'all-RecType' => 'RecType',
@@ -929,7 +913,7 @@ your bug as I make changes.
 
 You can find documentation for this module with the perldoc command.
 
-    perldoc IPDR::Collection::CiscoSSL
+    perldoc IPDR::Cisco
 
 You can also look for information at:
 
@@ -937,19 +921,19 @@ You can also look for information at:
 
 =item * AnnoCPAN: Annotated CPAN documentation
 
-L<http://annocpan.org/dist/IPDR-Collection-CiscoSSL>
+L<http://annocpan.org/dist/IPDR-Collection-Cisco>
 
 =item * CPAN Ratings
 
-L<http://cpanratings.perl.org/d/IPDR-Collection-CiscoSSL>
+L<http://cpanratings.perl.org/d/IPDR-Collection-Cisco>
 
 =item * RT: CPAN's request tracker
 
-L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=IPDR-Collection-CiscoSSL>
+L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=IPDR-Collection-Cisco>
 
 =item * Search CPAN
 
-L<http://search.cpan.org/dist/IPDR-Collection-CiscoSSL>
+L<http://search.cpan.org/dist/IPDR-Collection-Cisco>
 
 =back
 
@@ -957,11 +941,11 @@ L<http://search.cpan.org/dist/IPDR-Collection-CiscoSSL>
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2011 Andrew S. Kennedy, all rights reserved.
+Copyright 2009 Andrew S. Kennedy, all rights reserved.
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
 
 =cut
 
-1; # End of IPDR::Collection::CiscoSSL
+1; # End of IPDR::Collection::Cisco
